#########################################
#   CRITICAL PRELIMINARY REQUIREMENTS
# 1. Download MixRegLS: https://hedeker-sites.uchicago.edu/sites/hedeker.uchicago.edu/files/uploads/MixregLS_RevisedSept2013.zip
# 2. Unzip MIXREGLS.zip folder (critical files in the folder: mixregls.dll, mixregls_function.r, & mixregls_run.r)
# 3. Note directory for unzipped MIXREGLS folder
#########################################
  
#Install Formula (required to run MixRegLS)
#install.packages(c("Formula","LaplacesDemon"))
#library(Formula)
#library(LaplacesDemon)

###############################################################
###            Define MACRO Variable Values Below           ###
###      Use Calculate_Values.xlsx to Determine Values      ###
###############################################################

#Folder Directory for (unzipped) MixRegLS (don't forget the second forward slash!)
  MIXREGLS = "E:/MIXREGLS/"
#Individuals
  N = 50
#Repeated Occasions
  ni = 15
#Location-Model Fixed Intercept (Outcome Scale)
  beta0 = 0
#Location-Model Random Intercept Variance (Outcome Scale)
  b0var = 2
#Scale-Model Fixed Intercept (Log Scale)
  tau0 = 0.643
#Scale-Model Random Intercept Variance
  t0var = 0.10
#Correlation between Location- and Scale-Model Random Intercept
  b0t0corr = 0.30
#Scale-Model Fixed Predictor Effect (Outcome Scale)
  tau1 = 0.133
#Pseudo-R2
  R2 = 0.10
#Is the Predictor Continuous? (1=Yes, 0=No)
  Continuous = 1
    #Mean of Predictor W (=0 if W is continuous)
      Wmean = 0
    #Variance of Continuous Predictor W
    #Does not apply to binary predictor, but must have 
    #non-null value after equal sign for program to run
      Wvar = 0.563
#Number of Replications for Power Analysis
  Nreps = 1000
  
################################################
###      !DO NOT CHANGE ANYTHING BELOW!      ###
###   Create Data, Run MIXREGLS, Get Power   ###
################################################
  
#Load MIXREGLS dll (Make sure you're using a 32-bit version of R)
#You may get a warining that DLL attempted to change FPU from 8001f to 9001f (no worries...)
dyn.load(paste(MIXREGLS,"mixregls.dll",sep=""))
is.loaded("mixreglsr")
source(paste(MIXREGLS,"mixregls_function.r",sep=""))  

#Power with Clopper-Pearson CIs
ClopPears <- function(x,n,alpha){
  if (x == 0) {
    ll <- 0
    ul <- 1 - (alpha/2)^(1/n)
  }
  else if (x == n) {
    ll <- (alpha/2)^(1/n)
    ul <- 1
  }
  else {
    ll <- 1/(1 + (n - x + 1) / (x * qf(alpha/2, 2 * x, 2 * (n-x+1))))
    ul <- 1/(1 + (n - x) / ((x + 1) * qf(1-alpha/2, 2 * (x+1), 2 *
                                           (n-x))))
  }
  c(ll,ul)
}

#Create empty Results Data Frame
Results = data.frame(Rep=numeric(),
                     beta0_NP=numeric(),    
                     b0var_NP=numeric(),
                     tau0_NP=numeric(),
                     t0sd_NP=numeric(),
                     b0var_NP_DS=numeric(),
                     b0var_YP=numeric(),
                     beta0_YP=numeric(),
                     b0var_YP_DS=numeric(),
                     tau0_YP=numeric(),
                     tau1_YP=numeric(),
                     t0var_NP=numeric(),
                     t0var_YP=numeric(),
                     PseudoR2=numeric(),
                     LMSMRIcorr_YP=numeric(),
                     RLonWS_NP=numeric(),
                     RLonWS_YP=numeric(),
                     LMSMRIcov_NP=numeric(),
                     LMSMRIcov_YP=numeric(),
                     LMSMRIcorr_NP=numeric(),
                     tau1_SE_YP=numeric(),
                     tau1_Lower_YP=numeric(), 
                     tau1_Upper_YP=numeric(),
                     tau1_SS_YP=numeric(),
                     t0sd_YP=numeric(),
                     Neg2LL_NP=numeric(),
                     Neg2LL_YP=numeric())

#Calculate SMRI variance for simulation
t0var_sim = t0var*(1 - R2)
                   
for (i in 1:Nreps){
  
  #Create Data
  MELS = NULL
  rho = b0t0corr
  for (PersonID in 1:N){
    #Generate Standard Normal Variates with Specified Correlation
    b0i = rnorm(1,mean=0,sd=1)
    t0i = rho*b0i + sqrt(1 - rho**2)*rnorm(1,mean=0,sd=1)
    #Transform Intercepts to have (Prior) Mean of 0 with Specified Variance
    b0i = 0 + sqrt(b0var)*b0i
    t0i = 0 + sqrt(t0var_sim)*t0i
    #Generate Predictor Value
    if (Continuous == 0){
      W = rbern(1,Wmean)
    }
    if (Continuous == 1){
      W = rnorm(1,mean=Wmean,sd=sqrt(Wvar))
    }
    #Create Repeated Measures
    for (MeasureID in 1:ni){
      #Create Heterogeneous Residual Variances
      evar_i = exp((tau0 + t0i) + tau1*W)
      e_ti = rnorm(1,mean=0,sd=sqrt(evar_i))
      #Create Outcome Variable
      Y = (beta0 + b0i) + e_ti
      MELS=rbind(MELS,data.frame(PersonID, MeasureID, W, Y))
    }
  }
  
  if (b0t0corr == 0){
  
    #Run MixRegLS without Correlation - No Predictor
    mixregls.results_NP = mixregls.combined(Y ~ 1 | 1 | 1 , id="PersonID",data=MELS, NQ=15, NCOV=0)
  
    Results[i,"Rep"] = i
    Results[i,"beta0_NP"] = mixregls.results_NP$beta3
    Results[i,"b0var_NP"] = mixregls.results_NP$alpha3
    Results[i,"b0var_NP_DS"] = exp(mixregls.results_NP$alpha3)
    Results[i,"tau0_NP"] = mixregls.results_NP$tau3
    Results[i,"t0sd_NP"] = mixregls.results_NP$spar3
    Results[i,"t0var_NP"] = mixregls.results_NP$spar3**2
    Results[i,"Neg2LL_NP"] = mixregls.results_NP$dev3
  
    #Run MixRegLS without Correlation - Yes Predictor
    mixregls.results_YP = mixregls.combined(Y ~ 1 | 1 | W , id="PersonID",data=MELS, NQ=15, NCOV=0)
    
    Results[i,"beta0_YP"] = mixregls.results_YP$beta3
    Results[i,"b0var_YP"] = mixregls.results_YP$alpha3
    Results[i,"b0var_YP_DS"] = exp(mixregls.results_YP$alpha3)
    Results[i,"tau0_YP"] = mixregls.results_YP$tau3[1,1]
    Results[i,"tau1_YP"] = mixregls.results_YP$tau3[2,1]
    Results[i,"tau1_SE_YP"] = mixregls.results_YP$se3[4,1]
    Results[i,"tau1_Lower_YP"] = mixregls.results_YP$tau3[2,1] - 1.96*mixregls.results_YP$se3[4,1]
    Results[i,"tau1_Upper_YP"] = mixregls.results_YP$tau3[2,1] + 1.96*mixregls.results_YP$se3[4,1]
    if ((mixregls.results_YP$tau3[2,1] - 1.96*mixregls.results_YP$se3[4,1]) > 0) {
      Results[i,"tau1_SS_YP"] = 1
    } else if ((mixregls.results_YP$tau3[2,1] + 1.96*mixregls.results_YP$se3[4,1]) < 0) {
      Results[i,"tau1_SS_YP"] = 1
    } else {
      Results[i,"tau1_SS_YP"] = 0
    }
    Results[i,"PseudoR2"] = ((mixregls.results_NP$spar3**2 - mixregls.results_YP$spar3**2) / mixregls.results_NP$spar3**2)
    Results[i,"t0sd_YP"] = mixregls.results_YP$spar3
    Results[i,"t0var_YP"] = mixregls.results_YP$spar3**2
    Results[i,"Neg2LL_YP"] = mixregls.results_YP$dev3
    
  } else {
    
    #Run MixRegLS with correlation - No Predictor
    mixregls.results_NP = mixregls.combined(Y ~ 1 | 1 | 1 , id="PersonID",data=MELS, NQ=15, NCOV=1)
    
    Results[i,"Rep"] = i
    Results[i,"beta0_NP"] = mixregls.results_NP$beta3
    Results[i,"b0var_NP"] = mixregls.results_NP$alpha3
    Results[i,"b0var_NP_DS"] = exp(mixregls.results_NP$alpha3)
    Results[i,"tau0_NP"] = mixregls.results_NP$tau3
    Results[i,"t0sd_NP"] = mixregls.results_NP$spar3[2,1]
    Results[i,"RLonWS_NP"] = mixregls.results_NP$spar3[1,1]
    Results[i,"LMSMRIcov_NP"] = mixregls.results_NP$spar3[1,1]*sqrt(exp(mixregls.results_NP$alpha3))
    Results[i,"LMSMRIcorr_NP"] = (mixregls.results_NP$spar3[1,1]*sqrt(exp(mixregls.results_NP$alpha3))) / (sqrt(exp(mixregls.results_NP$alpha3)*(mixregls.results_NP$spar3[1,1]**2 + mixregls.results_NP$spar3[2,1]**2)))
    Results[i,"t0var_NP"] = mixregls.results_NP$spar3[1,1]**2 + mixregls.results_NP$spar3[2,1]**2
    Results[i,"Neg2LL_NP"] = mixregls.results_NP$dev3
    
    #Run MixRegLS with correlation - Yes Predictor
    mixregls.results_YP = mixregls.combined(Y ~ 1 | 1 | W , id="PersonID",data=MELS, NQ=15, NCOV=1)
    
    Results[i,"beta0_YP"] = mixregls.results_YP$beta3
    Results[i,"b0var_YP"] = mixregls.results_YP$alpha3
    Results[i,"b0var_YP_DS"] = exp(mixregls.results_YP$alpha3)
    Results[i,"tau0_YP"] = mixregls.results_YP$tau3[1,1]
    Results[i,"tau1_YP"] = mixregls.results_YP$tau3[2,1]
    Results[i,"tau1_SE_YP"] = mixregls.results_YP$se3[4,1]
    Results[i,"tau1_Lower_YP"] = mixregls.results_YP$tau3[2,1] - 1.96*mixregls.results_YP$se3[4,1]
    Results[i,"tau1_Upper_YP"] = mixregls.results_YP$tau3[2,1] + 1.96*mixregls.results_YP$se3[4,1]
    if ((mixregls.results_YP$tau3[2,1] - 1.96*mixregls.results_YP$se3[4,1]) > 0) {
      Results[i,"tau1_SS_YP"] = 1
    } else if ((mixregls.results_YP$tau3[2,1] + 1.96*mixregls.results_YP$se3[4,1]) < 0) {
      Results[i,"tau1_SS_YP"] = 1
    } else {
      Results[i,"tau1_SS_YP"] = 0
    }
    Results[i,"PseudoR2"] = ((mixregls.results_NP$spar3[1,1]**2 + mixregls.results_NP$spar3[2,1]**2) - (mixregls.results_YP$spar3[1,1]**2 + mixregls.results_YP$spar3[2,1]**2)) / (mixregls.results_NP$spar3[1,1]**2 + mixregls.results_NP$spar3[2,1]**2)
    Results[i,"t0sd_YP"] = mixregls.results_YP$spar3[2,1]
    Results[i,"RLonWS_YP"] = mixregls.results_YP$spar3[1,1]
    Results[i,"LMSMRIcov_YP"] = mixregls.results_YP$spar3[1,1]*sqrt(exp(mixregls.results_YP$alpha3))
    Results[i,"LMSMRIcorr_YP"] = (mixregls.results_YP$spar3[1,1]*sqrt(exp(mixregls.results_YP$alpha3))) / (sqrt(exp(mixregls.results_YP$alpha3)*(mixregls.results_YP$spar3[1,1]**2 + mixregls.results_YP$spar3[2,1]**2)))
    Results[i,"t0var_YP"] = mixregls.results_YP$spar3[1,1]**2 + mixregls.results_YP$spar3[2,1]**2
    Results[i,"Neg2LL_YP"] = mixregls.results_YP$dev3
  }
}

#Calculate Parameter Recovery
#Find problem esimates
Results = subset(Results, t0sd_NP > 0 & PseudoR2 > 0 & PseudoR2 < 1, select=Rep:Neg2LL_YP)
ParamRecov = data.frame(Obs=numeric(),Mean=numeric(),Lower95CI=numeric(),Upper95CI=numeric(),
                        Pctl25=numeric(),Pctl50=numeric(),Pctl75=numeric())

if (b0t0corr == 0){

  for (j in 8:14) {
    ParamRecov[(j - 7),1] = Nreps
    ParamRecov[(j - 7),2] = mean(Results[,j])
    ParamRecov[(j - 7),3] = mean(Results[,j]) - 1.96*(sd(Results[,j])/sqrt(Nreps))
    ParamRecov[(j - 7),4] = mean(Results[,j]) + 1.96*(sd(Results[,j])/sqrt(Nreps))
    ParamRecov[(j - 7),5] = quantile(Results[,j], c(.25))
    ParamRecov[(j - 7),6] = quantile(Results[,j], c(.50))
    ParamRecov[(j - 7),7] = quantile(Results[,j], c(.75))
  }
  
  rownamesobj = c("beta0: LM Fixed Intercept (Outcome Scale)",
                  "b0var_YP_DS: LM RI Variance (Outcome Scale) - Yes Predictor",
                  "tau0_YP: SM Fixed Intercept (Log Scale) - Yes Predictor",
                  "tau1_YP: SM Fixed Effect of Predictor W (Log Scale)",
                  "t0var_NP: SM RI Variance - No Predictor",
                  "t0var_YP: SM RI Variance - Yes Predictor",
                  "PseudoR2: Proportion Reduction SM RI Variance from Predictor W")
  rownames(ParamRecov) = rownamesobj
  
} else {
  
  for (j in 8:15) {
    ParamRecov[(j - 7),1] = Nreps
    ParamRecov[(j - 7),2] = mean(Results[,j])
    ParamRecov[(j - 7),3] = mean(Results[,j]) - 1.96*(sd(Results[,j])/sqrt(Nreps))
    ParamRecov[(j - 7),4] = mean(Results[,j]) + 1.96*(sd(Results[,j])/sqrt(Nreps))
    ParamRecov[(j - 7),5] = quantile(Results[,j], c(.25))
    ParamRecov[(j - 7),6] = quantile(Results[,j], c(.50))
    ParamRecov[(j - 7),7] = quantile(Results[,j], c(.75))
  }
  
  rownamesobj = c("beta0: LM Fixed Intercept (Outcome Scale)",
                  "b0var: LM RI Variance (Outcome Scale) - Yes Predictor",
                  "tau0: SM Fixed Intercept (Log Scale) - Yes Predictor",
                  "tau1: SM Fixed Effect of Predictor W (Log Scale)",
                  "t0var: SM RI Variance - No Predictor",
                  "t0var: SM RI Variance - Yes Predictor",
                  "PseudoR2: Proportion Reduction SM RI Variance from Predictor W",
                  "Correlation b/w LM and SM RI - Yes Predictor")
  rownames(ParamRecov) = rownamesobj
}

sig = sum(Results$tau1_SS_YP == 1)
CI = ClopPears(sig,Nreps,0.05)
CI = as.matrix(CI)
Power = data.frame(Power=numeric(), Lower95CI=numeric(), Upper95CI=numeric())
Power[1,1] = mean(Results$tau1_SS_YP)
Power[1,2] = CI[1,1]
Power[1,3] = CI[2,1]

#Provide Parameter Recovery Statistics
options(digits=3)
ParamRecov
#Power Estimates with 95% Clopper-Pearson CI
options(digits=3)
Power